/*
 *   COPYRIGHT NOTICE: This code was written by Markus Mummert. All
 *   rights reserved. FOR LICENCE TERMS, SEE ACCOMPANYING FILE "LICENCE".
 */
/************************************************************************/
/*									*/
/*  Datei    :	HMAX.C							*/
/*  Version  :	8.8.98/4						*/
/*  Autor    :	Markus Mummert						*/
/*  Kommentar:	Funktionen fuer normierte max. Filterimpulsantworthoehe	*/
/*		und deren normierter Zeitpunkt in Abhaengigkeit von der	*/
/*		Filtercodenummer (passend (!) zu FILTFLP 8.8.98/10);	*/
/*		Abfragefunktion Einpolfilter. Daten fuer HNMAXTAB	*/
/*		durch Simulation (Realiserung der Filter aus FILTFLP	*/
/*		bei einem F3DB/FSAMP-Verhaeltnis von 1e-4 und Ermitteln	*/
/*		des hoechsten Abtastwertes).				*/
/*									*/
/*  History  :	Erstversion vermutlich Anfang 1994			*/
/*		10.2.94/2,Mum	lauffaehig mit Funktion hnmax()		*/
/*		29.7.94/3,Mum	Einbau Funktion tnmax()			*/
/*		8.8.98/4,Mum	passt zu FILTFLP 13.5.91/8 - 8.8.98/10	*/
/*		    Simulations-Shell-Skript fuer Programm LHBP am Ende	*/
/*									*/
/************************************************************************/

#include <stdio.h>
#include <string.h>
#include "filtflp.h"

typedef struct {
	int FILTCODE;
	char *FABR;
	double TNMAX, HNMAX;
} HNMAXTAB;

/* Uebertragen aus filtflp und Ergaenzt um hnmax-Wert aus Skript HNMAX */
static HNMAXTAB hnmaxtab[] = {
	1, "P1",  0.0000, 1.0000,	/* s.a. isp1() unten	*/
	2, "2P1", 0.6440, 0.5716,
	14,"3P1", 1.0204, 0.5309,
	15,"4P1", 1.3053, 0.5151,
	3, "8P1", 2.1064, 0.4953,
	4, "B2",  0.8237, 0.5136,
	17,"B3",  1.4288, 0.4650,
	8, "B4",  1.9044, 0.4504,
	5, "B8",  3.1344, 0.4538,
	11,"PG2", 0.7358, 0.5409,
	19,"PG3", 1.2334, 0.4960,
	13,"PG4", 1.6270, 0.4800,
	10,"PG8", 2.6823, 0.4742,
	12,"P2",  1.1115, 0.4559,
	18,"P3",  2.0549, 0.4044,
	7, "P4",  2.8947, 0.3816,
	16,"P8",  5.8917, 0.3373,
	0, "", 0.0, 0.0
};

/* Rueckabe ungleich null wenn Filtertyp p1 */
int isp1(filtcode)
int filtcode;
{
	return(filtcode == 1);
}

/* Rueckabe tnmax */
double tnmax(filtcode)
int filtcode;
{
	int i;

	for (i = 0; hnmaxtab[i].FILTCODE != 0; i++) {
		if (g_ftyptab[i].FTYPCODE == 0 ||
		    strcmp(g_ftyptab[i].FTYPABR,hnmaxtab[i].FABR)) {
			fprintf(stderr,
	    "(intern) hnmax: Tabelleninkonsistenz zu filtflp\n");
			exit(-1);
		}
		if (filtcode == hnmaxtab[i].FILTCODE) {
			return(hnmaxtab[i].TNMAX);
		}
	}
	fprintf(stderr, "(intern) hnmax: filtcode unbekannt\n");
	exit(-1);
}

/* Rueckabe hnmax */
double hnmax(filtcode)
int filtcode;
{
	int i;

	for (i = 0; hnmaxtab[i].FILTCODE != 0; i++) {
		if (g_ftyptab[i].FTYPCODE == 0 ||
		    strcmp(g_ftyptab[i].FTYPABR,hnmaxtab[i].FABR)) {
			fprintf(stderr,
	    "(intern) hnmax: Tabelleninkonsistenz zu filtflp\n");
			exit(-1);
		}
		if (filtcode == hnmaxtab[i].FILTCODE) {
			return(hnmaxtab[i].HNMAX);
		}
	}
	fprintf(stderr, "(intern) hnmax: filtcode unbekannt\n");
	exit(-1);
}
/*******hnmax*************************************************************
#
# hnmax - Maximum (tnmax, hnmax) der normierten Impulsantwort hn(t)
#
# Mum, 29.7.94/3
#
# Es gilt hn=h(n*T*w3dB)/(T*w3dB), w3dB=f3dB*2*pi, dirac.128=32767*d(n)
# also hnmax=max{h(nmax*T*2*pi*f3dB)}/(f3dB*2*pi*T*32767)
#      tnmax=nmax*T*2*pi*f3dB
#
# cat $0; exit
	
# Unsicherheit etwa in in der 4ten Nachkommastelle 
F3DB=1.2800e+00
FSAMP=1.2800e+04

PARMS=`echo "
ifile           /users/mummert/signals/dirac2s.128 
ifmt            word
iscale          1.0000e+00
fsamp           $FSAMP
filttype        lp
f3db            $F3DB
phase           0.0000e+00
oscale          1.0000e+00
ofile           -
ofmt            ascii
"`

for i in P1 2P1 3P1 4P1 8P1 PG2 PG3 PG4 PG8 B2 B3 B4 B8 P2 P3 P4 P8
do
	# Absolutes Maximum suchen
	HMAX=`lhbp filtcode $i $PARMS | awk 'BEGIN { max = -1E33 }
	{
		if ($1 > max)
			max = $1
	}
	END { print max }'`
	lhbp filtcode $i $PARMS | awk 'BEGIN { k = 0; nsum = 0 }
	{
		# Abtastwertnummer mitteln, weil Maximum durch mangelnde
		# Quantisierung ueber mehere Abtastwerte laeuft
		if ((($1 - hmax)/hmax < 1e-4) && ((hmax - $1)/hmax < 1e-4)) {
			nsum += NR
			k++
		#	print fc,NR,$1
		}
	}
	END {
			n = nsum/k
			zpi = 2*3.1415926
			tnmax=n*zpi*f3db/fs
			# hnmax=hmax/205.8811
			hnmax=hmax/(zpi*f3db*32767/fs)
			if (fc == "P1")
				printf "%s	%6.4lf	%6.4lf	# von Hand, %6.4lf %6.4lf schlecht kond. wg. Spitze\n",fc,0,1,tnmax,hnmax
			else
				printf "%s	%6.4lf	%6.4lf\n",fc,tnmax,hnmax
	}' fc=$i hmax=$HMAX fs=$FSAMP f3db=$F3DB:
done
#
# Ergebnisse
# ----------
#
# Code	tnmax	hnmax
# 
# P1	0.000	1.000	# von Hand, 0.0012 0.9994 schlecht kond. wg. Spitze
# 2P1	0.6440	0.5716
# 3P1	1.0204	0.5309
# 4P1	1.3053	0.5151
# 8P1	2.1064	0.4953
# PG2	0.7358	0.5409
# PG3	1.2334	0.4960
# PG4	1.6270	0.4800
# PG8	2.6823	0.4742
# B2	0.8237	0.5136
# B3	1.4288	0.4650
# B4	1.9044	0.4504
# B8	3.1344	0.4538
# P2	1.1115	0.4559
# P3	2.0549	0.4044
# P4	2.8947	0.3816
# P8	5.8917	0.3373
#
********Dateiende********************************************************/

