/*
 *   COPYRIGHT NOTICE: This code was written by Markus Mummert. All
 *   rights reserved. FOR LICENCE TERMS, SEE ACCOMPANYING FILE "LICENCE".
 */
/************************************************************************/
/*									*/
/*  Datei    :	PORTAB.C 						*/
/*  Version  :	24.2.99/9						*/
/*  Autor    :	Markus Mummert						*/
/*									*/
/*  Kommentar:	Include-Datei zur Cross-Platform-Portabilitaet.		*/
/*		Zusammen mit PORTAB.H zu verwenden. Siehe Infos dort.	*/
/*									*/
/************************************************************************/

#include <stdio.h>
#include "portab.h"

#ifdef WIN_COMPATIBILITY_FUNCTIONS
void srand48(x) long x; { srand((unsigned int)x); }
double drand48() { return(rand()/(RAND_MAX+1.0)); }
#endif

#ifdef UNNATURAL_BYTEORDER

/*----------------------------------------------------------------------*/
/*  FREAD	fuert die I/O-Funktion fread() durch. Sollten WORDs	*/
/*		oder LONGs angefordert worden sein, wird 		*/
/*		ein Byteswap und ggf. Wordswap gemacht, um eine		*/
/*		INTEL/MOTOROLA Formatwandlung zu erreichen.		*/
/*----------------------------------------------------------------------*/
int FREAD(b, s, n, f)
char *b;
int s, n;
FILE *f;
{
	int retval;

	switch(s) {
	case sizeof(WORD):
		retval = fread(b, s, n, f);
		swapbytesinword(b, n);
		return(retval);
	case sizeof(LONG):
		retval = fread(b, s, n, f);
		swapbytesinword(b, 2*n);
		swapwordsinlong(b, n);
		return(retval);
	default:
		return(fread(b, s, n, f));
	}
}
/*----------------------------------------------------------------------*/
/*  FWRITE	macht zunaechst Byteswap und ggf. Wordswap im Puffer,	*/
/*		sofern WORD oder LONG geschrieben werden sollen,um eine	*/
/*		INTEL/MOTOROLA Formatwandlung zu erreichen.		*/
/*		Danach wird die I/O-Funktion fwrite() aufgerufen. 	*/
/*----------------------------------------------------------------------*/
int FWRITE(b, s, n, f)
char *b;
int s, n;
FILE *f;
{
	switch(s) {
	case sizeof(WORD):
		swapbytesinword(b, n);
		return(fwrite(b, s, n, f));
	case sizeof(LONG):
		swapbytesinword(b, 2*n);
		swapwordsinlong(b, n);
		return(fwrite(b, s, n, f));
	default:
		return(fwrite(b, s, n, f));
	}
}
/*----------------------------------------------------------------------*/

swapbytesinword(b ,n)
register BYTE *b;
register int n;
{
	register BYTE x;
	register int i;

	for (i = 0; i < n; i++) {
		x = *(b+2*i);
		*(b+2*i) = *(b+2*i+1);
		*(b+2*i+1) = x;
	}
}

swapwordsinlong(b, n)
register WORD *b;
register int n;
{
	register WORD x;
	register int i;

	for (i = 0; i < n; i++) {
		x = *(b+2*i);
		*(b+2*i) = *(b+2*i+1);
		*(b+2*i+1) = x;
	}
}

#else
int dummy_portab_c;			/* some dont like empty files	*/
#endif					/* UNNATURAL_BYTEORDER		*/

/*******Dateiende********************************************************/

