/*
 *   COPYRIGHT NOTICE: This code was written by Markus Mummert. All
 *   rights reserved. FOR LICENCE TERMS, SEE ACCOMPANYING FILE "LICENCE".
 */
/************************************************************************
 *
 * $Source: /home/mummert/src/SWMUMDIS/spsy/RCS/spsy.c,v $
 * $Revision: 0.8 $
 * $Date: 1999/07/25 01:27:17 $
 * $State: Exp $
 *
 * SPSY.C (Hauptprogramm)
 *
 * Synthese des zeitvarianten FTT-Spektrums mit Phasen
 *
 * Autor: Markus Mummert
 *
 * Anmerkung: Die tc... Datenabteilungen von TVF betreffen das
 * die Parameter auf der Analyse-Seite des Spektrums. Historisch
 * haben sie den Praefix tc, weil sie sich bei TVF im Kontext
 * von CONTSY speziell auf die Analyse-Seite der Zeitkonturen
 * beziehen. 
 *
 * $Author: mummert $
 * $Log: spsy.c,v $
 * Revision 0.8  1999/07/25 01:27:17  mummert
 * Automatische Pegelskalierung verbessert; outdelay-Berechnung koorigiert,
 * Laufzeitangabe (ca.) auch von Analyse, dennoch DELAY-Option stillgelegt,
 * da abhaengig von NTI auf Analyseseite; Default NTI=16; AB3DB und AFILTCODE
 * erfordern explizte Angaben, Uebername von SB3DB und SFILTCODE mehr.
 *
 * Revision 0.7  1999/02/25 22:45:21  mummert
 * New freeware-licence; module portab with MS-Visual-C++ support
 *
 * Revision 0.6  1998/10/20 00:25:31  mummert
 * Nochmals verbesserte Portabilitaet, Module PORTAB und FILTBANK;
 * geaenderte KEYTAB-Defaults
 *
 * Revision 0.5  1998/10/14 01:06:13  mummert
 * Weiter verbesserte Portabilitaet
 *
 * Revision 0.4  1998/10/11 23:46:47  mummert
 * Verbesserung Portabilitaet
 *
 * Revision 0.3  1998/08/28 23:44:29  mummert
 * ERB- und SPINC-Skala
 *
 * Revision 0.2  1998/08/28 00:50:22  mummert
 * Geaendertes Modul KEY mit DFLTPARMPATH; DFLTPARMFILE nun spsy.par
 *
 * Revision 0.1  1998/08/13 00:03:16  mummert
 * Erstversion
 *
 * Vorgeschichte: contsy.c Revision 1.4  1998/08/09 00:46:53  mummert
 *
 ************************************************************************/
	
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "portab.h"			/* Unix/Atari Portabilitaet	*/
#include "key.h"			/* Keytab-Management-Modul	*/
#include "tvf.h"			/* TVK-Modul			*/

#define	DFLTPARMFILE	"spsy.par"	/* Default-Parameterdateiname	*/
#define MAXNTT 10240			/* Maximale Anzahl von Daten	*/
					/* pro Auswerteintervall	*/
#define sqr(x)		((x)*(x))
#define gem(x,y)	(sqrt(sqr(x)+sqr(y)))
#ifndef PI
#define PI		3.14159265358979323846
#endif
#ifndef ZPI
#define ZPI		(2*PI)
#endif

/*----------------------------------------------------------------------*/
/*	globale Variable						*/
/*----------------------------------------------------------------------*/

unsigned BYTE	g_spltab[MAXNTT],	/* FTT-Pegelspektrum		*/
		g_spptab[MAXNTT];	/* FTT-Phasenspektrum		*/

GTVFPARMS g_gfpv;			/* TVF-Parameter (s. tvf.h/c)	*/
GTVFSTAT g_gfsv;			/* TVF-Status			*/

char 	g_splfn[MAXLINE],		/* Dateiname Pegel/Phasen-Sp.	*/
 	g_sppfn[MAXLINE],
	g_outfn[MAXLINE],		/* Ausgangsdatei		*/
	g_verbose[MAXLINE],		/* Kontrollmeldungsflag		*/
	g_delay[MAXLINE],		/* Ausgabeverzoegerung		*/
	g_ipmode[MAXLINE];		/* Flag Puls statt Treppe	*/

char	g_unitdfs[MAXLINE],		/* Hilfsstrings, da diese Para-	*/
	g_unitbs[MAXLINE],		/* meter nicht direkt durch 	*/
	g_sfilts[MAXLINE],		/* Keytab-Mechanismus in g_gfpv	*/
	g_tcfilt[MAXLINE];		/* gesetzt werden koennen	*/

int	g_nti,				/* Auswerteintervall		*/
        g_tcgrid;                       /* Kanalfreigabeindexmodulo	*/

double	g_ooffs = 0;			/* Gesamtoffset dB		*/
	
FILE 	*g_outfp,			/* Filepointer			*/
	*g_splfp,
	*g_sppfp;
					/* Ablaufleit-Flags,		*/
int	g_verbflag   = 0,		/* Kontrollmeldungen aus,	*/
	g_delflag    = 0,		/* Ausgabeverzoegerung aus,	*/
	g_ipflag     = 0;		/* Impulsmode			*/

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
					/* Keytab (vergl. key.h/c)	*/
KEY_TAB kt[]= {
"FSAMP",	pt(g_gfpv.fa),	DOUBLE,	"Abtastfrequenz Zeitsignal in Hz",
				"12800","1000 100000",		
"NTI",		pt(g_nti),	INT,
				"Signalabtastw. pro Auswerteinterv.",
				"16",	"1 10000",
"SPLFILE",	g_splfn,	STRING,	"Datei FTT-Pegelspektrum (ftt-Format)",
				"-",	EMPTY,
"SPPFILE",	g_sppfn,	STRING,	"Datei FTT-(Bandpass-)Phasenspektrum",
				"-",	EMPTY,
"SPGRID",	pt(g_tcgrid),	INT,
				"f-Abstand Spektrum in Rasterfrequenzen",
				"5",	"1 1000",
"NFI",		pt(g_gfpv.nfi),	INT,	"Anzahl Rasterfrequenzen",
				"380",	"1 5000",
"F0",		pt(g_gfpv.f0), 	DOUBLE,	"unterste Rasterfrequenz in Hz",
				"20",	"20 20000",
"UNITDF",	g_unitdfs,	STRING,	"Einheit Rasterfrequenzabstand",
				"bark",
				"hz prozent bark erb spinc h b p e s",
"DF",		pt(g_gfpv.df),	DOUBLE,	"Masszahl Rasterfrequenzabstand",
				".05",	"0.001 20000",
"UNITB",	g_unitbs,	STRING,
				"Einheit 3dB-Bandbreite Analyse/Synthese",
				"bark",
				"hz prozent bark erb spinc h b p e s",
"SB3DB",	pt(g_gfpv.b),	DOUBLE,
				"Masszahl Synthese-3dB-Bandbreite",
				"0.7",	"0.001 20000",
"SFILTCODE",	g_sfilts,	STRING, "Synthesefiltertyp",
				"b4",
	    "p1 p2 p3 p4 p8 2p1 3p1 4p1 8p1 b2 b3 b4 b8 pg2 pg3 pg4 pg8",
"STGCOMP",	pt(g_gfpv.tgcomp),
				DOUBLE,	"Synthesefilter Tg-Kompensationsfaktor",
				"0.9",	"0 10",
"AB3DB",	pt(g_gfpv.tcb),	DOUBLE,
				"Masszahl Analyse-B3dB",
				"0.3",	"1E-32 20000",
"AFILTCODE",	g_tcfilt,	STRING,
				"Analysefiltertyp",
				"4p1",
	    "p1 p2 p3 p4 p8 2p1 3p1 4p1 8p1 b2 b3 b4 b8 pg2 pg3 pg4 pg8",
"IPMODE",	g_ipmode,	STRING,
				"Auswertezeitpuls statt Treppe (NTI>1)",
				"nein",	"ja nein j n",
"OFILE",	g_outfn,	STRING, "Zeitsignal-Ausgangsdatei",
				"-",	EMPTY,
"OOFFS",	pt(g_ooffs), 	DOUBLE,	
				"Ausgangspegeloffset/dB (-6dB ist sicher)",
				"-6.0",	"-100 100",
"CHECK",	g_verbose,	STRING,	"Kontrollmeldungen ja/nein",
				"nein",	"ja nein j n",
/* "DELAY",	g_delay,	STRING,	
 * 				"Ausgabeverzoeg. um komp. Laufzt.",
 *				"nein",	"ja nein j n",
 */
0L,0L,0L,0L,0L,0L };

/*----------------------------------------------------------------------*/
/*  main								*/
/*----------------------------------------------------------------------*/
main(argc, argv)
int argc;
char *argv[];
{
	char *errmsg, *etp;
					/* Keytab-Mechanismus		*/
	if (initkeytab(argc, argv, kt, DFLTPARMFILE))
		exit(-1);
					/* INTERNE DEFAULT-PARAMETER	*/
	g_gfsv.allpass= 0;		/* LZ-Feinkorr. zunaechts aus	*/
	g_gfpv.syntadj= 1.;		/* Synthesezeitpunktjustage	*/
	g_gfpv.scale  = 1.;		/* Syntheseverstaerkung		*/

					/* Analyseparameter-Defaults	*/
       	g_gfpv.tcdf = g_gfpv.df;
/*	if (fabs(g_gfpv.tcb) < 1E-64)
 *	        g_gfpv.tcb = g_gfpv.b;
 *	if (g_tcfilt[0] == '-') 
 *		strcpy(g_tcfilt, g_sfilts);
 */					/* NACHKONVERTIERUNG von 	*/
					/* Keytabparametern in Rechen-	*/
					/* parameter			*/
	g_gfpv.unitdf = unitcode(g_unitdfs);
	g_gfpv.unitb  = unitcode(g_unitbs);
	g_gfpv.tcunitb = unitcode(g_unitbs);
	g_gfpv.tcunitdf = unitcode(g_unitdfs);
	if ((g_gfpv.tcfiltcode = filtercode(g_tcfilt)) <= 0 ||
	    (g_gfpv.sfiltcode = filtercode(g_sfilts)) <= 0 ) {
		errorhandler("(intern) Filtercode unbekannt");
		exit(-1);
	}
	g_gfpv.idxmod = g_tcgrid;	/* TVF-Rechenraster = ..GRID	*/
					/* Ablaufleit-Flags		*/
	if (g_ipmode[0] == 'j')
		g_ipflag = 1;
/*	if (g_delay[0] == 'j')
 *		g_delflag = 1;
 */
	if (g_verbose[0] == 'j')
		g_verbflag = 1;

					/* EINGANGSDATEIEN ...		*/
	etp = NULL;
	if (g_splfn[0] != '-' || g_splfn[1] != 0) {
	    if ((g_splfp = fopen(g_splfn, READ)) == NULL) 
		etp = g_splfn;
	} else {
		errorhandler("Datei FTT-Pegelspektrum nicht spezifiziert");
		exit(-1);
	}
	if (g_sppfn[0] != '-' || g_sppfn[1] != 0) {
	    if ((g_sppfp = fopen(g_sppfn, READ)) == NULL)
		etp = g_sppfn;
	} else {
		errorhandler("Datei FTT-Phasenspektrum nicht spezifiziert");
		exit(-1);
	}

	if (etp != NULL) {
		errmsg = catmsg(etp, "kann nicht oeffnen");
		errorhandler(errmsg);
		exit(-1);
	}
					/* AUSGANGSDATEI...		*/
	if (g_outfn[0] != '-') {
		if ((g_outfp = fopen(g_outfn, WRITE)) == NULL) {
			errmsg = catmsg(g_outfn, "kann nicht oeffnen");
			errorhandler(errmsg);
			exit(-1);
		}
	} else {
		errorhandler("keine Ausgangsdatei spezifiziert");
		exit(-1);
	}
					/* Speicheranwahl, Initialis.	*/
	if (init_tvf(&g_gfpv,&g_gfsv)){	/* TVF-interner Parameter	*/
		errorhandler("Speichermangel");
		exit(-1);
	}
	if (g_verbflag) {
		check();
	}
					/* RECHNUNG			*/
					/* Schleife bis Signaldateiende	*/
	if (do_calc()) {
		errmsg = catmsg(g_outfn, "kann nicht schreiben");
		errorhandler(errmsg);
		exit(-1);
	}
					/* Clean-Up			*/
	if (g_splfp != NULL) fclose(g_splfp);
	if (g_sppfp != NULL) fclose(g_sppfp);
	fclose(g_outfp);
	dispose_tvf(&g_gfsv);
	exit(0);
}
/*----------------------------------------------------------------------*/
/*  do_calc		Hauptrechenschleife 				*/
/*			
/*----------------------------------------------------------------------*/
int do_calc()
{
	WORD input, outbuff;
	int k, i, nsp, error, outdelay;
	double scale, effb, effhmax;

	/*
	 * SKALIERUNGSFAKTOR scale
	 *
	 * effektive 3dB-Bandbreite effB der Verkettung Analyse- und
	 * Synthesefilter wird dominiert von der Bandbreite des schmaleren
	 * Filters. Dies realisert effB=1/sqrt(1/(BA*BA)+1/(BS*BS)).
	 * Bei gleicher Breite erschient effb um 1/sqrt(2) verkleinert.
	 * Dies entspricht der Situation bei Filtertypen nP1, PGn, Bn.
	 */
	effb = 1. / gem(1./g_gfpv.b, 1./g_gfpv.tcb);
	/*
	 * Die Hoehe der Impulsantwort effhmax der Verkettung von Analyse-
	 * und Synthesefilter wird auch vom schmaleren Filter dominiert.
	 * Ansatz daher als Wichtung mit den Bandbreiten. Da die Hnmax
	 * der nP1, PGn und Bn sowieso alle um 0.5 liegen, passiert
	 * hier nichts. (Ausser bei P1, dort Hnmax=1. Bei zweimal
	 * P1 hinternander mit gleicher Bandbreite wird in der
	 * Realitaet ein effHnmax=0.5 daraus ... dann wird der Pegel
	 * um 6dB zu klein sein.
	 */
	effhmax = gem(hnmax(g_gfpv.sfiltcode)/g_gfpv.b,
		      hnmax(g_gfpv.tcfiltcode)/g_gfpv.tcb) * effb ;
	/*
	 * In scale geht also ein: der gewuenschte Pegeloffset, bei
	 * IPMODE=ja ein Gewicht entsprechend der samples/Auswerteintervall,
	 * der efffektive Kanalabstand, und eben effb und effhmax
	 */
	scale = (exp(log((double)10.)*g_ooffs/20.)) * 
	  (g_ipflag ? g_nti : 1) *
	  g_gfpv.df * g_gfpv.idxmod / effb / effhmax / 3.1;

					/* effektive Anzahl der Kanaele	*/ 
	nsp = (g_gfpv.nfi + g_gfpv.idxmod - 1) / g_gfpv.idxmod;
					/* ggf. Ausgabeverzoegerung in	*/
					/* Abtw. um kompensierte Laufzt	*/
	if (g_delflag)
	    outdelay = (int)(g_gfsv.tgmax * g_gfpv.fa * g_gfpv.tgcomp);
	else
	    outdelay = 0;	
					/* Schleife			*/
					/* ueber Auswerteintervalle	*/
	while (-1) {
					/* Schleife ueber Abtastwerte	*/
					/* eines Auswerteintervalls	*/
	    for (i = 0; i < g_nti; i++) {
					/* TVF-Eingangsmodulatoren rech.*/
		spmo_tvf(&g_gfpv, &g_gfsv);

		if (i == 0) {		/* bei 1. Abtw. d. AWI...	*/
		    
		    FREAD(g_spltab, sizeof(unsigned BYTE), nsp, g_splfp); 
		    if (feof(g_splfp) || ferror(g_splfp))
		        return(0);
		    FREAD(g_spptab, sizeof(unsigned BYTE), nsp, g_sppfp); 
		    if (feof(g_sppfp) || ferror(g_sppfp))
		        return(0);
		    insert_sp(&g_gfpv, &g_gfsv, i, scale);
		} else if (!g_ipflag) {
		    insert_sp(&g_gfpv, &g_gfsv, i, scale);
		}
					/* Synthesefilter und Ausgangs-	*/
					/* modulatoren rechnen		*/
		synt_tvf(&g_gfpv, &g_gfsv, &outbuff);

					/* Ausgangsabtastwert auf Datei	*/
					/*   Ausgabe verzoegert um	*/
					/*   kompensierte Laufzeit	*/
		if (outdelay <= 0) {
		    FWRITE(&outbuff,sizeof(WORD),1,g_outfp);
		    if (ferror(g_outfp)) 
			return(-1);
		} else
		    outdelay--;
	    }
	}
}
/*----------------------------------------------------------------------*/
/*  check	Kontrollmeldungen					*/
/*----------------------------------------------------------------------*/
check()
{
printf("\n");
printf("    hoechste Rasterfrequenz: %0.2lf Hz\n", 
	        (g_gfsv.parmtab)[g_gfpv.nfi-1].FREQ);
printf("    Grundlaufzeit Analyse  : ca. %d Abtastwerte\n",
		(int)(tnmax(g_gfpv.tcfiltcode) * g_gfpv.fa /
                ((g_gfsv.parmtab)[0].TCAIM / hnmax(g_gfpv.tcfiltcode))));
printf("    Grundlaufzeit Synthese : ca. %d Abtastwerte\n",
		(int)(g_gfsv.tgmax * g_gfpv.fa * g_gfpv.tgcomp));
printf("    \n");
printf("\n");
}
/*----------------------------------------------------------------------*/
/*  header	Hilfstext-Ueberschrift, wird von key.c gerufen.		*/
/*----------------------------------------------------------------------*/
header()
{
char string[90], version[90];

sscanf("$Revision: 0.8 $ $Date: 1999/07/25 01:27:17 $ $Author: mummert $",
	"$%*s %s $ $%*s %s %*s $ $%*s %s $",
	string, string+30, string+60);
sprintf(version, "SPSY V%s %s %s", string, string+30, string+60);
fprintf(stderr,"\n");
fprintf(stderr,"    %s\n", version);
fprintf(stderr,"\n");
fprintf(stderr,"    Synthese des zeitvarianten FTT-Spektrums mit Phasen nach Mummert 1991\n");
fprintf(stderr,"\n");
}
/*----------------------------------------------------------------------*/
/*  unitcode	gibt Einheitencode von Einheiteingabe zurueck, so, wie	*/
/*		fuer tvf-Modul benoetigt.				*/
/*----------------------------------------------------------------------*/
int unitcode(uk)
char *uk;
{
	switch(uk[0] | 0x20) {
		default :
		case 'h':	return(HZ);
		case 'p':	return(PROZ);
		case 'b':	return(BARK);
		case 'e':	return(ERB);
		case 's':	return(SPINC);
	}
}
/*----------------------------------------------------------------------*/
/*  filtercode	gibt Filtercode von Filterkuerzeleingabe FK zurueck,	*/
/*		so, wie	fuer tvf-Modul benoetigt.			*/
/*----------------------------------------------------------------------*/
int filtercode(fk)
char *fk;
{
	int i, j, code;
	char string[MAXLINE+1], *abr, *name, *comment;

	i = 0;
	while(!info_tvf(i++,&code,&abr,&name,&comment)) {
	    j = 0;
	    while (abr[j] != 0) {
		if ((abr[j] | 0x20) != (fk[j] | 0x20)) 
			break;
		j++;
	    }
	    if (abr[j] == 0 && fk[j] == 0)
		break;
	}
	return(code);
}
/*----------------------------------------------------------------------*/
/*	INSERT_SP	Einbau der Spektralwerte in die TVF-Status-	*/
/*			tabellenfelder RS und IS durch Addition.	*/
/*			scale ist Masstabsfaktor.			*/
/*			step gibt die nummer des Abtastwertes im Aus-	*/
/*			werteintervall an, beginnend bei 0.		*/
/*----------------------------------------------------------------------*/
insert_sp(gfp, gfs, step, scale)
GTVFPARMS *gfp;
GTVFSTAT *gfs;
int step;
double scale;
{
        int i, k;
	AFREQSTAT *stat;
	AFREQPARM *parm;
	SINOUT si, co;
	double c, amp, level, log10 = log((double)10.);
	unsigned WORD phase;

	stat = gfs->stattab;
	parm = gfs->parmtab;
	k=0;
	for (i = 0; i < gfp->nfi; i += gfp->idxmod) {
		if (g_spltab[k] > 0) {
			level = 0.5*g_spltab[k];
			amp = exp(log10*(level-84.)/20.) * scale / 
			  ARITHM_ZH15M1;
		} else {
			amp = 0;
		}	
		/*
		 * Umrechnung Bandfilterphase aus Datei auf Spektrumphase:
		 *
		 * Falls NTI=1 ist immer step=0, MPH dreht negativ -> +.
		 * Falls nicht, wird die Funktion fuer die selben
		 * Bandpassphase-Werte NTI-mal mit step=0 ... NTI-1 
		 * aufgerufen. Der Bandpassphasenwert muss daher mit
		 * step*MDP erst in den tatsaechlichen Wert umgerechnet
		 * werden (Phasenregel bei Treppenstufeninterpolation
		 * des Spektrums).
		 */
		phase = g_spptab[k] * 256 + 
		  step * parm[i].MDP +
		  stat[i].MPH;
		SINUSCOSINUS(phase, si, co);

		stat[i].RS += amp * co;
		stat[i].IS += amp * si;
		k++;
	}
}
/*******Ende*************************************************************/
